// LightDlg.cpp : implementation file
//

#include "stdafx.h"
#include "Stage.h"
#include "LightDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLightDlg dialog

CLightDlg::CLightDlg(CWnd* pParent, C3dLight* pLight)
	: CDialog(CLightDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CLightDlg)
	m_iBlue = 0;
	m_iGreen = 0;
	m_iRed = 0;
	m_dConstant = 0.0;
	m_dLinear = 0.0;
	m_dQuadratic = 0.0;
	m_dRange = 0.0;
	//}}AFX_DATA_INIT
	m_pLight = pLight;

	Create(CLightDlg::IDD, pParent);
}


void CLightDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CLightDlg)
	DDX_Control(pDX, IDC_RED, m_sbRed);
	DDX_Control(pDX, IDC_GREEN, m_sbGreen);
	DDX_Control(pDX, IDC_BLUE, m_sbBlue);
	DDX_Scroll(pDX, IDC_BLUE, m_iBlue);
	DDX_Scroll(pDX, IDC_GREEN, m_iGreen);
	DDX_Scroll(pDX, IDC_RED, m_iRed);
	DDX_Text(pDX, IDC_CONSTANT, m_dConstant);
	DDX_Text(pDX, IDC_LINEAR, m_dLinear);
	DDX_Text(pDX, IDC_QUADRATIC, m_dQuadratic);
	DDX_Text(pDX, IDC_RANGE, m_dRange);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CLightDlg, CDialog)
	//{{AFX_MSG_MAP(CLightDlg)
	ON_WM_DESTROY()
	ON_WM_TIMER()
	ON_WM_VSCROLL()
	ON_BN_CLICKED(IDC_APPLY, OnApply)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CLightDlg::PostNcDestroy() 
{
	delete this;
}

BOOL CLightDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	// Set the scrollbar ranges
	m_sbRed.SetScrollRange(0, 255);
	m_sbGreen.SetScrollRange(0, 255);
	m_sbBlue.SetScrollRange(0, 255);
	
	// Show the current values
	ShowValues();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CLightDlg::OnDestroy() 
{
	CDialog::OnDestroy();
}

void CLightDlg::OnTimer(UINT nIDEvent) 
{
	ShowValues();
}

void CLightDlg::ShowValues()
{
	if (!m_pLight) return;

	double r, g, b;
	m_pLight->GetColor(r, g, b);
	m_pLight->GetAttenuation(m_dConstant, m_dLinear, m_dQuadratic);
	m_dRange = m_pLight->GetRange();
	m_iRed = 255 - (int) (r * 255);
	m_iGreen = 255 - (int) (g * 255);
	m_iBlue = 255 - (int) (b * 255);

	UpdateData(FALSE);
}

void CLightDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	int iPos = pScrollBar->GetScrollPos();
	switch (nSBCode) {
	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		iPos = nPos;
		break;

	case SB_LINEDOWN:
		iPos += 1;
		break;

	case SB_PAGEDOWN:
		iPos += 25;

	case SB_LINEUP:
		iPos -= 1;
		break;

	case SB_PAGEUP:
		iPos -= 25;

	default:
		break;
	}
	if (iPos < 0) {
		iPos = 0;
	} else if (iPos > 255) {
		iPos = 255;
	}
	pScrollBar->SetScrollPos(iPos);

	NewColor();
}

void CLightDlg::NewColor()
{
	// Get the scroll bar values
	UpdateData(TRUE);
	ASSERT(m_pLight);
	m_pLight->SetColor((255 - m_iRed) / 255.0,
					   (255 - m_iGreen) / 255.0,
					   (255 - m_iBlue) / 255.0);
}

void CLightDlg::OnApply() 
{
	// Update the attenuation and range paramters
	UpdateData(TRUE);
	ASSERT(m_pLight);
	m_pLight->SetRange(m_dRange);
	m_pLight->SetAttenuation(m_dConstant, m_dLinear, m_dQuadratic);
}
